from setuptools import setup

setup(name='blackjack_am',
      version='0.2',
      description='Blackjack (21) single player game',
      packages=['blackjack_am'],
      author='Alcia Moncrieffe',
      author_email='alcia.moncrieffe@gmail.com',
      zip_safe=False)
